@@ -8,18 +8,11 @@ import time
 from rasa.core.brokers.broker import EventBroker
 from rasa.shared.utils.io import DEFAULT_ENCODING
 from rasa.utils.endpoints import EndpointConfig
-from rasa.shared.exceptions import RasaException
 
 logger = logging.getLogger(__name__)
 
 
-class KafkaProducerInitializationError(RasaException):
-    """Raised if the Kafka Producer cannot be properly initialized."""
-
-
 class KafkaEventBroker(EventBroker):
-    """Kafka event broker."""
-
     def __init__(
         self,
         url: Union[Text, List[Text], None],
@@ -27,7 +20,6 @@ class KafkaEventBroker(EventBroker):
         client_id: Optional[Text] = None,
         sasl_username: Optional[Text] = None,
         sasl_password: Optional[Text] = None,
-        sasl_mechanism: Optional[Text] = "PLAIN",
         ssl_cafile: Optional[Text] = None,
         ssl_certfile: Optional[Text] = None,
         ssl_keyfile: Optional[Text] = None,
@@ -51,10 +43,6 @@ class KafkaEventBroker(EventBroker):
                 logging with respect to producer group administration.
             sasl_username: Username for plain authentication.
             sasl_password: Password for plain authentication.
-            sasl_mechanism: Authentication mechanism when security_protocol is
-                configured for SASL_PLAINTEXT or SASL_SSL.
-                Valid values are: PLAIN, GSSAPI, OAUTHBEARER, SCRAM-SHA-256,
-                SCRAM-SHA-512. Default: `PLAIN`
             ssl_cafile: Optional filename of ca file to use in certificate
                 verification.
             ssl_certfile: Optional filename of file in pem format containing
@@ -77,7 +65,6 @@ class KafkaEventBroker(EventBroker):
         self.security_protocol = security_protocol.upper()
         self.sasl_username = sasl_username
         self.sasl_password = sasl_password
-        self.sasl_mechanism = sasl_mechanism
         self.ssl_cafile = ssl_cafile
         self.ssl_certfile = ssl_certfile
         self.ssl_keyfile = ssl_keyfile
@@ -136,18 +123,28 @@ class KafkaEventBroker(EventBroker):
         import kafka
 
         if self.security_protocol == "PLAINTEXT":
-            authentication_params = dict(
-                security_protocol=self.security_protocol, ssl_check_hostname=False,
+            self.producer = kafka.KafkaProducer(
+                client_id=self.client_id,
+                bootstrap_servers=self.url,
+                value_serializer=lambda v: json.dumps(v).encode(DEFAULT_ENCODING),
+                security_protocol=self.security_protocol,
+                ssl_check_hostname=False,
             )
         elif self.security_protocol == "SASL_PLAINTEXT":
-            authentication_params = dict(
+            self.producer = kafka.KafkaProducer(
+                client_id=self.client_id,
+                bootstrap_servers=self.url,
+                value_serializer=lambda v: json.dumps(v).encode(DEFAULT_ENCODING),
                 sasl_plain_username=self.sasl_username,
                 sasl_plain_password=self.sasl_password,
-                sasl_mechanism=self.sasl_mechanism,
+                sasl_mechanism="PLAIN",
                 security_protocol=self.security_protocol,
             )
         elif self.security_protocol == "SSL":
-            authentication_params = dict(
+            self.producer = kafka.KafkaProducer(
+                client_id=self.client_id,
+                bootstrap_servers=self.url,
+                value_serializer=lambda v: json.dumps(v).encode(DEFAULT_ENCODING),
                 ssl_cafile=self.ssl_cafile,
                 ssl_certfile=self.ssl_certfile,
                 ssl_keyfile=self.ssl_keyfile,
@@ -155,32 +152,24 @@ class KafkaEventBroker(EventBroker):
                 security_protocol=self.security_protocol,
             )
         elif self.security_protocol == "SASL_SSL":
-            authentication_params = dict(
+            self.producer = kafka.KafkaProducer(
+                client_id=self.client_id,
+                bootstrap_servers=self.url,
+                value_serializer=lambda v: json.dumps(v).encode(DEFAULT_ENCODING),
                 sasl_plain_username=self.sasl_username,
                 sasl_plain_password=self.sasl_password,
-                sasl_mechanism=self.sasl_mechanism,
                 ssl_cafile=self.ssl_cafile,
                 ssl_certfile=self.ssl_certfile,
                 ssl_keyfile=self.ssl_keyfile,
                 ssl_check_hostname=self.ssl_check_hostname,
                 security_protocol=self.security_protocol,
+                sasl_mechanism="PLAIN",
             )
         else:
             raise ValueError(
                 f"Cannot initialise `KafkaEventBroker`: "
                 f"Invalid `security_protocol` ('{self.security_protocol}')."
             )
-        try:
-            self.producer = kafka.KafkaProducer(
-                client_id=self.client_id,
-                bootstrap_servers=self.url,
-                value_serializer=lambda v: json.dumps(v).encode(DEFAULT_ENCODING),
-                **authentication_params,
-            )
-        except AssertionError as e:
-            raise KafkaProducerInitializationError(
-                f"Cannot initialise `KafkaEventBroker`: {e}"
-            )
 
     def _publish(self, event) -> None:
         logger.debug(f"Calling kafka send({self.topic}, {event})")
