@@ -1,13 +1,9 @@
-import datetime
-
 import tensorflow as tf
 import tensorflow_addons as tfa
 import numpy as np
 import logging
-import os
-import shutil
+import random
 from collections import defaultdict
-from pathlib import Path
 from typing import (
     List,
     Text,
@@ -15,23 +11,14 @@ from typing import (
     Tuple,
     Union,
     Optional,
-    Callable,
-    TYPE_CHECKING,
     Any,
 )
 
-from tqdm import tqdm
-from rasa.constants import CHECKPOINT_MODEL_NAME
-from rasa.shared.utils.io import is_logging_disabled
-import rasa.utils.io
 from rasa.utils.tensorflow.model_data import RasaModelData, FeatureSignature
 from rasa.utils.tensorflow.constants import (
     SEQUENCE,
     SENTENCE,
-    TENSORBOARD_LOG_LEVEL,
     RANDOM_SEED,
-    TENSORBOARD_LOG_DIR,
-    CHECKPOINT_MODEL,
     EMBEDDING_DIMENSION,
     REGULARIZATION_CONSTANT,
     SIMILARITY_TYPE,
@@ -55,41 +42,43 @@ from rasa.utils.tensorflow.constants import (
     CONCAT_DIMENSION,
     DROP_RATE_ATTENTION,
     SCALE_LOSS,
+    LEARNING_RATE,
     CONSTRAIN_SIMILARITIES,
     MODEL_CONFIDENCE,
 )
 from rasa.utils.tensorflow import layers
 from rasa.utils.tensorflow.transformer import TransformerEncoder
-
-if TYPE_CHECKING:
-    from tensorflow.python.ops.summary_ops_v2 import ResourceSummaryWriter
+from rasa.utils.tensorflow.temp_keras_modules import TmpKerasModel
+from rasa.utils.tensorflow.data_generator import (
+    RasaDataGenerator,
+    RasaBatchDataGenerator,
+)
+from tensorflow.python.keras.utils import tf_utils
 
 logger = logging.getLogger(__name__)
 
 
-TENSORBOARD_LOG_LEVELS = ["epoch", "minibatch"]
-
-
 # noinspection PyMethodOverriding
-class RasaModel(tf.keras.models.Model):
-    """Completely override all public methods of keras Model.
+class RasaModel(TmpKerasModel):
+    """Abstract custom Keras model.
 
-    Cannot be used as tf.keras.Model
+     This model overwrites the following methods:
+    - train_step
+    - test_step
+    - predict_step
+    - save
+    - load
+    Cannot be used as tf.keras.Model.
     """
 
-    def __init__(
-        self,
-        random_seed: Optional[int] = None,
-        tensorboard_log_dir: Optional[Text] = None,
-        tensorboard_log_level: Optional[Text] = "epoch",
-        checkpoint_model: Optional[bool] = False,
-        **kwargs,
-    ) -> None:
+    def __init__(self, random_seed: Optional[int] = None, **kwargs,) -> None:
         """Initialize the RasaModel.
 
         Args:
             random_seed: set the random seed to get reproducible results
         """
+        # make sure that keras releases resources from previously trained model
+        tf.keras.backend.clear_session()
         super().__init__(**kwargs)
 
         self.total_loss = tf.keras.metrics.Mean(name="t_loss")
@@ -97,55 +86,16 @@ class RasaModel(tf.keras.models.Model):
 
         self._training = None  # training phase should be defined when building a graph
 
-        self._predict_function = None
-
         self.random_seed = random_seed
+        self._set_random_seed()
 
-        self.tensorboard_log_dir = tensorboard_log_dir
-        self.tensorboard_log_level = tensorboard_log_level
+        self._tf_predict_step = None
+        self.prepared_for_prediction = False
 
-        self.train_summary_writer = None
-        self.test_summary_writer = None
-        self.model_summary_file = None
-        self.tensorboard_log_on_epochs = True
-
-        self.best_metrics_so_far = {}
-        self.checkpoint_model = checkpoint_model
-        self.best_model_file = None
-        self.best_model_epoch = -1
-        if self.checkpoint_model:
-            model_checkpoint_dir = rasa.utils.io.create_temporary_directory()
-            self.best_model_file = os.path.join(
-                model_checkpoint_dir, f"{CHECKPOINT_MODEL_NAME}.tf_model"
-            )
-
-    def _set_up_tensorboard_writer(self) -> None:
-        if self.tensorboard_log_dir is not None:
-            if self.tensorboard_log_level not in TENSORBOARD_LOG_LEVELS:
-                raise ValueError(
-                    f"Provided '{TENSORBOARD_LOG_LEVEL}' "
-                    f"('{self.tensorboard_log_level}') "
-                    f"is invalid! Valid values are: {TENSORBOARD_LOG_LEVELS}"
-                )
-            self.tensorboard_log_on_epochs = self.tensorboard_log_level == "epoch"
-
-            current_time = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
-            class_name = self.__class__.__name__
-
-            train_log_dir = (
-                f"{self.tensorboard_log_dir}/{class_name}/{current_time}/train"
-            )
-            test_log_dir = (
-                f"{self.tensorboard_log_dir}/{class_name}/{current_time}/test"
-            )
-
-            self.train_summary_writer = tf.summary.create_file_writer(train_log_dir)
-            self.test_summary_writer = tf.summary.create_file_writer(test_log_dir)
-
-            self.model_summary_file = (
-                f"{self.tensorboard_log_dir}/{class_name}/{current_time}"
-                f"/model_summary.txt"
-            )
+    def _set_random_seed(self):
+        random.seed(self.random_seed)
+        tf.random.set_seed(self.random_seed)
+        np.random.seed(self.random_seed)
 
     def batch_loss(
         self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
@@ -171,7 +121,7 @@ class RasaModel(tf.keras.models.Model):
 
     def batch_predict(
         self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
-    ) -> Dict[Text, tf.Tensor]:
+    ) -> Dict[Text, Union[tf.Tensor, Dict[Text, tf.Tensor]]]:
         """Predicts the output of the given batch.
 
         Args:
@@ -182,114 +132,19 @@ class RasaModel(tf.keras.models.Model):
         """
         raise NotImplementedError
 
-    def fit(
-        self,
-        model_data: RasaModelData,
-        epochs: int,
-        batch_size: Union[List[int], int],
-        evaluate_on_num_examples: int,
-        evaluate_every_num_epochs: int,
-        batch_strategy: Text,
-        silent: bool = False,
-        loading: bool = False,
-        eager: bool = False,
-    ) -> None:
-        """Fit model data."""
-        # don't setup tensorboard writers when training during loading
-        if not loading:
-            self._set_up_tensorboard_writer()
-
-        tf.random.set_seed(self.random_seed)
-        np.random.seed(self.random_seed)
-
-        disable = silent or is_logging_disabled()
-
-        evaluation_model_data = None
-        if evaluate_on_num_examples > 0:
-            if not disable:
-                logger.info(
-                    f"Validation accuracy is calculated every "
-                    f"{evaluate_every_num_epochs} epochs."
-                )
-
-            model_data, evaluation_model_data = model_data.split(
-                evaluate_on_num_examples, self.random_seed
-            )
-
-        (
-            train_dataset_function,
-            tf_train_on_batch_function,
-        ) = self._get_tf_train_functions(eager, model_data, batch_strategy)
-        (
-            evaluation_dataset_function,
-            tf_evaluation_on_batch_function,
-        ) = self._get_tf_evaluation_functions(eager, evaluation_model_data)
-
-        val_results = {}  # validation is not performed every epoch
-        progress_bar = tqdm(range(epochs), desc="Epochs", disable=disable)
-
-        training_steps = 0
-
-        for epoch in progress_bar:
-            epoch_batch_size = self.linearly_increasing_batch_size(
-                epoch, batch_size, epochs
-            )
-
-            training_steps = self._batch_loop(
-                train_dataset_function,
-                tf_train_on_batch_function,
-                epoch_batch_size,
-                True,
-                training_steps,
-                self.train_summary_writer,
-            )
-
-            if self.tensorboard_log_on_epochs:
-                self._log_metrics_for_tensorboard(epoch, self.train_summary_writer)
-
-            postfix_dict = self._get_metric_results()
-
-            if evaluate_on_num_examples > 0:
-                if self._should_evaluate(evaluate_every_num_epochs, epochs, epoch):
-                    self._batch_loop(
-                        evaluation_dataset_function,
-                        tf_evaluation_on_batch_function,
-                        epoch_batch_size,
-                        False,
-                        training_steps,
-                        self.test_summary_writer,
-                    )
-
-                    if self.tensorboard_log_on_epochs:
-                        self._log_metrics_for_tensorboard(
-                            epoch, self.test_summary_writer
-                        )
-
-                    val_results = self._get_metric_results(prefix="val_")
-                    self._save_model_checkpoint(
-                        current_results=val_results, epoch=epoch
-                    )
-
-                postfix_dict.update(val_results)
-
-            progress_bar.set_postfix(postfix_dict)
-
-        if self.checkpoint_model:
-            logger.info(
-                f"The model of epoch {self.best_model_epoch} "
-                f"(out of {epochs} in total) will be stored!"
-            )
-        if self.model_summary_file is not None:
-            self._write_model_summary()
-
-        self._training = None  # training phase should be defined when building a graph
-        if not disable:
-            logger.info("Finished training.")
-
-    def train_on_batch(
+    def train_step(
         self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
-    ) -> None:
-        """Train on batch."""
+    ) -> Dict[Text, float]:
+        """Performs a train step using the given batch.
+
+        Args:
+            batch_in: The batch input.
+
+        Returns:
+            Training metrics.
+        """
+        self._training = True
+
         # calculate supervision and regularization losses separately
         with tf.GradientTape(persistent=True) as tape:
             prediction_loss = self.batch_loss(batch_in)
@@ -322,53 +177,121 @@ class RasaModel(tf.keras.models.Model):
 
         self.optimizer.apply_gradients(zip(gradients, self.trainable_variables))
 
-    def build_for_predict(
-        self, predict_data: RasaModelData, eager: bool = False
-    ) -> None:
-        self._training = False  # needed for tf graph mode
-        self.prepare_for_predict()
-        self._predict_function = self._get_tf_call_model_function(
-            predict_data.as_tf_dataset, self.batch_predict, eager, "prediction"
-        )
+        self._training = None
 
-    def predict(self, predict_data: RasaModelData) -> Dict[Text, tf.Tensor]:
-        if self._predict_function is None:
-            logger.debug("There is no tensorflow prediction graph.")
-            self.build_for_predict(predict_data)
+        return self._get_metric_results()
 
-        # Prepare a single batch of the size of the input
-        batch_in = predict_data.prepare_batch()
+    def test_step(
+        self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
+    ) -> Dict[Text, float]:
+        """Tests the model using the given batch.
 
-        self._training = False  # needed for eager mode
-        return self._predict_function(batch_in)
+        This method is used during validation.
+
+        Args:
+            batch_in: The batch input.
+
+        Returns:
+            Testing metrics.
+        """
+        self._training = False
+
+        prediction_loss = self.batch_loss(batch_in)
+        regularization_loss = tf.math.add_n(self.losses)
+        total_loss = prediction_loss + regularization_loss
+        self.total_loss.update_state(total_loss)
+
+        self._training = None
+
+        return self._get_metric_results()
+
+    def predict_step(
+        self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
+    ) -> Dict[Text, tf.Tensor]:
+        """Predicts the output for the given batch.
+
+        Args:
+            batch_in: The batch to predict.
+
+        Returns:
+            Prediction output.
+        """
+        self._training = False
+
+        if not self.prepared_for_prediction:
+            # in case the model is used for prediction without loading, e.g. directly
+            # after training, we need to prepare the model for prediction once
+            self.prepare_for_predict()
+            self.prepared_for_prediction = True
+
+        return self.batch_predict(batch_in)
+
+    @staticmethod
+    def _dynamic_signature(
+        batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
+    ) -> List[List[tf.TensorSpec]]:
+        element_spec = []
+        for tensor in batch_in:
+            if len(tensor.shape) > 1:
+                shape = [None] * (len(tensor.shape) - 1) + [tensor.shape[-1]]
+            else:
+                shape = [None]
+            element_spec.append(tf.TensorSpec(shape, tensor.dtype))
+        # batch_in is a list of tensors, therefore we need to wrap element_spec into
+        # the list
+        return [element_spec]
+
+    def rasa_predict(self, model_data: RasaModelData) -> Dict[Text, tf.Tensor]:
+        """Custom prediction method that builds tf graph on the first call.
+
+        Args:
+            model_data: The model data to use for prediction.
+
+        Return:
+            Prediction output.
+        """
+        self._training = False
+        if not self.prepared_for_prediction:
+            # in case the model is used for prediction without loading, e.g. directly
+            # after training, we need to prepare the model for prediction once
+            self.prepare_for_predict()
+            self.prepared_for_prediction = True
+
+        batch_in = RasaBatchDataGenerator.prepare_batch(model_data.data)
+
+        if self._run_eagerly:
+            return tf_utils.to_numpy_or_python_type(self.predict_step(batch_in))
+
+        if self._tf_predict_step is None:
+            self._tf_predict_step = tf.function(
+                self.predict_step, input_signature=self._dynamic_signature(batch_in)
+            )
+
+        return tf_utils.to_numpy_or_python_type(self._tf_predict_step(batch_in))
+
+    def _get_metric_results(self, prefix: Optional[Text] = "") -> Dict[Text, float]:
+        return {
+            f"{prefix}{metric.name}": metric.result()
+            for metric in self.metrics
+            if metric.name in self.metrics_to_log
+        }
 
     def save(self, model_file_name: Text, overwrite: bool = True) -> None:
+        """Save the model to the given file.
+
+        Args:
+            model_file_name: The file name to save the model to.
+            overwrite: If 'True' an already existing model with the same file name will
+                       be overwritten.
+        """
         self.save_weights(model_file_name, overwrite=overwrite, save_format="tf")
 
-    def copy_best(self, model_file_name: Text) -> None:
-        checkpoint_directory, checkpoint_file = os.path.split(self.best_model_file)
-        checkpoint_path = Path(checkpoint_directory)
-
-        # Copy all tf2 model files from the temp location to the final destination
-        for f in checkpoint_path.glob(f"{checkpoint_file}*"):
-            shutil.move(str(f.absolute()), model_file_name + f.suffix)
-
-        # Generate the tf2 checkpoint file, copy+replace to ensure consistency
-        destination_path, destination_file = os.path.split(model_file_name)
-        with open(os.path.join(checkpoint_directory, "checkpoint")) as in_file, open(
-            os.path.join(destination_path, "checkpoint"), "w"
-        ) as out_file:
-            for line in in_file:
-                out_file.write(line.replace(checkpoint_file, destination_file))
-
-        # Remove the old file
-        checkpoint_path.joinpath("checkpoint").unlink()
-
     @classmethod
     def load(
         cls,
         model_file_name: Text,
         model_data_example: RasaModelData,
+        predict_data_example: Optional[RasaModelData] = None,
         finetune_mode: bool = False,
         *args,
         **kwargs,
@@ -378,6 +301,8 @@ class RasaModel(tf.keras.models.Model):
         Args:
             model_file_name: Path to file containing model weights.
             model_data_example: Example data point to construct the model architecture.
+            predict_data_example: Example data point to speed up prediction during
+              inference.
             finetune_mode: Indicates whether to load the model for further finetuning.
             *args: Any other non key-worded arguments.
             **kwargs: Any other key-worded arguments.
@@ -386,184 +311,27 @@ class RasaModel(tf.keras.models.Model):
             Loaded model with weights appropriately set.
         """
         logger.debug(
-            f"Loading the model from {model_file_name} with finetune_mode={finetune_mode}..."
+            f"Loading the model from {model_file_name} "
+            f"with finetune_mode={finetune_mode}..."
         )
         # create empty model
         model = cls(*args, **kwargs)
+        learning_rate = kwargs.get("config", {}).get(LEARNING_RATE, 0.001)
         # need to train on 1 example to build weights of the correct size
-        model.fit(
-            model_data_example,
-            epochs=1,
-            batch_size=1,
-            evaluate_every_num_epochs=0,
-            evaluate_on_num_examples=0,
-            batch_strategy=SEQUENCE,
-            silent=True,  # don't confuse users with training output
-            loading=True,  # don't use tensorboard when doing a dummy fit run
-            eager=(
-                False if finetune_mode else True
-            ),  # load in eager mode only for prediction phase
-        )
+        model.compile(optimizer=tf.keras.optimizers.Adam(learning_rate))
+        data_generator = RasaBatchDataGenerator(model_data_example, batch_size=1)
+        model.fit(data_generator, verbose=False)
         # load trained weights
         model.load_weights(model_file_name)
 
+        # predict on one data example to speed up prediction during inference
+        # the first prediction always takes a bit longer to trace tf function
+        if not finetune_mode and predict_data_example:
+            model.rasa_predict(predict_data_example)
+
         logger.debug("Finished loading the model.")
         return model
 
-    def _total_batch_loss(
-        self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
-    ) -> tf.Tensor:
-        """Calculate total loss."""
-        prediction_loss = self.batch_loss(batch_in)
-        regularization_loss = tf.math.add_n(self.losses)
-        total_loss = prediction_loss + regularization_loss
-        self.total_loss.update_state(total_loss)
-
-        return total_loss
-
-    def _batch_loop(
-        self,
-        dataset_function: Callable,
-        call_model_function: Callable,
-        batch_size: int,
-        training: bool,
-        offset: int,
-        writer: Optional["ResourceSummaryWriter"] = None,
-    ) -> int:
-        """Run on batches."""
-        self.reset_metrics()
-
-        step = offset
-
-        self._training = training  # needed for eager mode
-        for batch_in in dataset_function(batch_size):
-            call_model_function(batch_in)
-
-            if not self.tensorboard_log_on_epochs:
-                self._log_metrics_for_tensorboard(step, writer)
-
-            step += 1
-
-        return step
-
-    @staticmethod
-    def _get_tf_call_model_function(
-        dataset_function: Callable,
-        call_model_function: Callable,
-        eager: bool,
-        phase: Text,
-    ) -> Callable:
-        """Convert functions to tensorflow functions."""
-        if eager:
-            return call_model_function
-
-        logger.debug(f"Building tensorflow {phase} graph...")
-
-        init_dataset = dataset_function(1)
-        tf_call_model_function = tf.function(
-            call_model_function, input_signature=[init_dataset.element_spec]
-        )
-        tf_call_model_function(next(iter(init_dataset)))
-
-        logger.debug(f"Finished building tensorflow {phase} graph.")
-
-        return tf_call_model_function
-
-    def _get_tf_train_functions(
-        self, eager: bool, model_data: RasaModelData, batch_strategy: Text
-    ) -> Tuple[Callable, Callable]:
-        """Create train tensorflow functions."""
-
-        def train_dataset_function(_batch_size: int) -> tf.data.Dataset:
-            return model_data.as_tf_dataset(_batch_size, batch_strategy, shuffle=True)
-
-        self._training = True  # needed for tf graph mode
-        return (
-            train_dataset_function,
-            self._get_tf_call_model_function(
-                train_dataset_function, self.train_on_batch, eager, "train"
-            ),
-        )
-
-    def _get_tf_evaluation_functions(
-        self, eager: bool, evaluation_model_data: Optional[RasaModelData]
-    ) -> Tuple[Optional[Callable], Optional[Callable]]:
-        """Create evaluation tensorflow functions."""
-        if evaluation_model_data is None:
-            return None, None
-
-        def evaluation_dataset_function(_batch_size: int) -> tf.data.Dataset:
-            return evaluation_model_data.as_tf_dataset(
-                _batch_size, SEQUENCE, shuffle=False
-            )
-
-        self._training = False  # needed for tf graph mode
-        return (
-            evaluation_dataset_function,
-            self._get_tf_call_model_function(
-                evaluation_dataset_function, self._total_batch_loss, eager, "evaluation"
-            ),
-        )
-
-    def _get_metric_results(self, prefix: Optional[Text] = None) -> Dict[Text, Text]:
-        """Get the metrics results"""
-        prefix = prefix or ""
-
-        return {
-            f"{prefix}{metric.name}": f"{metric.result().numpy():.3f}"
-            for metric in self.metrics
-            if metric.name in self.metrics_to_log
-        }
-
-    def _log_metrics_for_tensorboard(
-        self, step: int, writer: Optional["ResourceSummaryWriter"] = None
-    ) -> None:
-        if writer is not None:
-            with writer.as_default():
-                for metric in self.metrics:
-                    if metric.name in self.metrics_to_log:
-                        tf.summary.scalar(metric.name, metric.result(), step=step)
-
-    def _does_model_improve(self, current_results: Dict[Text, Text]) -> bool:
-        # Initialize best_metrics_so_far with the first results
-        if not self.best_metrics_so_far:
-            keys = filter(
-                lambda k: True if (k.endswith("_acc") or k.endswith("_f1")) else False,
-                current_results.keys(),
-            )
-            for key in keys:
-                self.best_metrics_so_far[key] = float(current_results[key])
-            return True
-
-        all_improved = all(
-            [
-                float(current_results[key]) > self.best_metrics_so_far[key]
-                for key in self.best_metrics_so_far.keys()
-            ]
-        )
-        if all_improved:
-            for key in self.best_metrics_so_far.keys():
-                self.best_metrics_so_far[key] = float(current_results[key])
-        return all_improved
-
-    def _save_model_checkpoint(
-        self, current_results: Dict[Text, Text], epoch: int
-    ) -> None:
-        if self.checkpoint_model and self._does_model_improve(current_results):
-            logger.debug(f"Creating model checkpoint at epoch={epoch + 1}...")
-            self.best_model_epoch = epoch + 1
-            self.save(self.best_model_file, overwrite=True)
-
-    @staticmethod
-    def _should_evaluate(
-        evaluate_every_num_epochs: int, epochs: int, current_epoch: int
-    ) -> bool:
-        return (
-            current_epoch == 0
-            or (current_epoch + 1) % evaluate_every_num_epochs == 0
-            or (current_epoch + 1) == epochs
-        )
-
     @staticmethod
     def batch_to_model_data_format(
         batch: Union[Tuple[tf.Tensor], Tuple[np.ndarray]],
@@ -576,118 +344,95 @@ class RasaModel(tf.keras.models.Model):
         data, shape) before, this method converts them into sparse tensors. Dense
         data is kept.
         """
+        # during training batch is a tuple of input and target data
+        # as our target data is inside the input data, we are just interested in the
+        # input data
+        if isinstance(batch[0], Tuple):
+            batch = batch[0]
+
         batch_data = defaultdict(lambda: defaultdict(list))
 
         idx = 0
         for key, values in data_signature.items():
             for sub_key, signature in values.items():
                 for is_sparse, feature_dimension, number_of_dimensions in signature:
+                    # we converted all 4D features to 3D features before
                     number_of_dimensions = (
                         number_of_dimensions if number_of_dimensions != 4 else 3
                     )
                     if is_sparse:
-                        # explicitly substitute last dimension in shape with known
-                        # static value
-                        shape = [
-                            batch[idx + 2][i] for i in range(number_of_dimensions - 1)
-                        ] + [feature_dimension]
-                        batch_data[key][sub_key].append(
-                            tf.SparseTensor(batch[idx], batch[idx + 1], shape)
+                        tensor, idx = RasaModel._convert_sparse_features(
+                            batch, feature_dimension, idx, number_of_dimensions
                         )
-                        idx += 3
                     else:
-                        if isinstance(batch[idx], tf.Tensor):
-                            batch_data[key][sub_key].append(batch[idx])
-                        else:
-                            # convert to Tensor
-                            batch_data[key][sub_key].append(
-                                tf.constant(batch[idx], dtype=tf.float32)
-                            )
-                        idx += 1
+                        tensor, idx = RasaModel._convert_dense_features(
+                            batch, feature_dimension, idx, number_of_dimensions
+                        )
+                    batch_data[key][sub_key].append(tensor)
 
         return batch_data
 
     @staticmethod
-    def linearly_increasing_batch_size(
-        epoch: int, batch_size: Union[List[int], int], epochs: int
-    ) -> int:
-        """Linearly increase batch size with every epoch.
+    def _convert_dense_features(
+        batch: Union[Tuple[tf.Tensor], Tuple[np.ndarray]],
+        feature_dimension: int,
+        idx: int,
+        number_of_dimensions: int,
+    ):
+        if isinstance(batch[idx], tf.Tensor):
+            # explicitly substitute last dimension in shape with known
+            # static value
+            if number_of_dimensions > 1 and (
+                batch[idx].shape is None or batch[idx].shape[-1] is None
+            ):
+                shape = [None] * (number_of_dimensions - 1) + [feature_dimension]
+                batch[idx].set_shape(shape)
 
-        The idea comes from https://arxiv.org/abs/1711.00489.
-        """
-        if not isinstance(batch_size, list):
-            return int(batch_size)
+            return batch[idx], idx + 1
 
-        if epochs > 1:
-            return int(
-                batch_size[0] + epoch * (batch_size[1] - batch_size[0]) / (epochs - 1)
-            )
-        else:
-            return int(batch_size[0])
-
-    def _write_model_summary(self):
-        total_number_of_variables = np.sum(
-            [np.prod(v.shape) for v in self.trainable_variables]
+        # convert to Tensor
+        return (
+            tf.constant(batch[idx], dtype=tf.float32, shape=batch[idx].shape),
+            idx + 1,
         )
-        layers = [
-            f"{layer.name} ({layer.dtype.name}) "
-            f"[{'x'.join(str(s) for s in layer.shape)}]"
-            for layer in self.trainable_variables
+
+    @staticmethod
+    def _convert_sparse_features(
+        batch: Union[Tuple[tf.Tensor], Tuple[np.ndarray]],
+        feature_dimension: int,
+        idx: int,
+        number_of_dimensions: int,
+    ):
+        # explicitly substitute last dimension in shape with known
+        # static value
+        shape = [batch[idx + 2][i] for i in range(number_of_dimensions - 1)] + [
+            feature_dimension
         ]
-        layers.reverse()
+        return tf.SparseTensor(batch[idx], batch[idx + 1], shape), idx + 3
 
-        with open(self.model_summary_file, "w") as file:
-            file.write("Variables: name (type) [shape]\n\n")
-            for layer in layers:
-                file.write(layer)
-                file.write("\n")
-            file.write("\n")
-            file.write(f"Total size of variables: {total_number_of_variables}")
+    def call(
+        self,
+        inputs: Union[tf.Tensor, List[tf.Tensor]],
+        training: Optional[tf.Tensor] = None,
+        mask: Optional[tf.Tensor] = None,
+    ) -> Union[tf.Tensor, List[tf.Tensor]]:
+        """Calls the model on new inputs.
 
-    def compile(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
+        Arguments:
+            inputs: A tensor or list of tensors.
+            training: Boolean or boolean scalar tensor, indicating whether to run
+              the `Network` in training mode or inference mode.
+            mask: A mask or list of masks. A mask can be
+                either a tensor or None (no mask).
 
-    def evaluate(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
-
-    def test_on_batch(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
-
-    def predict_on_batch(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
-
-    def fit_generator(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
-
-    def evaluate_generator(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
-
-    def predict_generator(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
-
-    def call(self, *args, **kwargs) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
-
-    def get_config(self) -> None:
-        raise Exception(
-            "This method should neither be called nor implemented in our code."
-        )
+        Returns:
+            A tensor if there is a single output, or
+            a list of tensors if there are more than one outputs.
+        """
+        # This method needs to be implemented, otherwise the super class is raising a
+        # NotImplementedError('When subclassing the `Model` class, you should
+        #   implement a `call` method.')
+        pass
 
 
 # noinspection PyMethodOverriding
@@ -700,11 +445,7 @@ class TransformerRasaModel(RasaModel):
         label_data: RasaModelData,
     ) -> None:
         super().__init__(
-            name=name,
-            random_seed=config[RANDOM_SEED],
-            tensorboard_log_dir=config[TENSORBOARD_LOG_DIR],
-            tensorboard_log_level=config[TENSORBOARD_LOG_LEVEL],
-            checkpoint_model=config[CHECKPOINT_MODEL],
+            name=name, random_seed=config[RANDOM_SEED],
         )
 
         self.config = config
@@ -713,7 +454,7 @@ class TransformerRasaModel(RasaModel):
 
         self._check_data()
 
-        label_batch = label_data.prepare_batch()
+        label_batch = RasaDataGenerator.prepare_batch(label_data.data)
         self.tf_label_data = self.batch_to_model_data_format(
             label_batch, self.label_signature
         )
@@ -1174,7 +915,7 @@ class TransformerRasaModel(RasaModel):
 
     def batch_predict(
         self, batch_in: Union[Tuple[tf.Tensor], Tuple[np.ndarray]]
-    ) -> Dict[Text, tf.Tensor]:
+    ) -> Dict[Text, Union[tf.Tensor, Dict[Text, tf.Tensor]]]:
         """Predicts the output of the given batch.
 
         Args:
